/***************************************************************************
                          kmasqauth.cpp  -  description
                             -------------------
    begin                : Wed Apr 18 2001
    copyright            : (C) 2001 by Stephan de Georgio
    email                : kmasqdialer@stephan.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "kmasqdialer.h"
#include "kmasqauth.h"
#include <qpushbutton.h>
#include <qframe.h>
#include <qlabel.h>
#include <qwhatsthis.h>
#include <klocale.h>

KMasqAuth::KMasqAuth(KMasqDialer *parent, const char *name ) : QDialog(parent,name, true)
{
	config = parent->returnConfig();
	config->setGroup("Options");

	setFixedSize(210, 150);
	setCaption(i18n("Authentication"));
	QFrame *frame = new QFrame(this);
	frame->setFrameStyle( QFrame::Box | QFrame::Sunken );
	frame->setGeometry(5, 5, width()-10, height()-40);
	
	QLabel *lblUsername = new QLabel(i18n("Username"), frame);
	lblUsername->setGeometry(5, 5, frame->width()-13, 15);
	username = new QLineEdit(frame);
	username->setGeometry(5, 20, frame->width()-13, 25);
	username->setText(config->readEntry("username", ""));
	QWhatsThis::add(username, i18n("The name of the user who is to be authenticated."));
	
	QLabel *lblPassword = new QLabel(i18n("Password"), frame);
	lblPassword->setGeometry(5, 45, frame->width()-13, 15);
	password = new QLineEdit(frame);
	password->setEchoMode(QLineEdit::Password);
	password->setGeometry(5, 60, frame->width()-13, 25);
	password->setText(config->readEntry("userpassword", ""));
	QWhatsThis::add(password, i18n("The password of the user who is to be authenticated."));
	
	save = new QCheckBox(i18n("Save password"), frame);
	save->setGeometry(5, 90, frame->width()-13, 15);
	save->setChecked(config->readBoolEntry("savepassword", false));
	QWhatsThis::add(save, i18n("Check this if you want your username to be stored in the config file - it is not encrypted."));
	
	QPushButton *Ok = new QPushButton(i18n("&Ok"), this);
	Ok->setDefault(true);
	Ok->setGeometry(width()-85, height()-30, 80, 25);
	connect(Ok, SIGNAL(clicked()), this, SLOT(slotOk()));
	
	QPushButton *Cancel = new QPushButton(i18n("&Cancel"), this);
	Cancel->setGeometry(width()-170, height()-30, 80, 25);
	connect(Cancel, SIGNAL(clicked()), this, SLOT(slotCancel()));
}

KMasqAuth::~KMasqAuth()
{
}

void KMasqAuth::slotOk()
{
	if(save->isChecked())
		config->writeEntry("userpassword", password->text());
	else
		config->writeEntry("userpassword", "");
	config->writeEntry("username", username->text());
	config->writeEntry("savepassword", save->isChecked());
	emit authenticated(username->text(), password->text());
	close(true);
}

void KMasqAuth::slotCancel()
{
	close(true);
}
