/***************************************************************************
                          kmasqclient.cpp  -  description
                             -------------------
    begin                : Sun Mar 25 2001
    copyright            : (C) 2001 by Stephan de Georgio
    email                : kmasqdialer@stephan.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmasqclient.h"
#include <klocale.h>

KMasqClient::KMasqClient(QObject *parent, const char *name)	: QObject( parent, name )
{
	connected = false;
  dialed_out = false;
	status = "";
  connection = new QSocket;
  connect(connection, SIGNAL(hostFound()), this, SLOT(slotHostFound()));
  connect(connection, SIGNAL(connected()), this, SLOT(slotConnected()));
  connect(connection, SIGNAL(connectionClosed()), this, SLOT(slotConnectionClosed()));
  connect(connection, SIGNAL(readyRead()), this, SLOT(slotReadyRead()));
  connect(connection, SIGNAL(error(int)), this, SLOT(slotError(int)));
}

KMasqClient::~KMasqClient()
{
}

/** Connect to the modem server */
void KMasqClient::ServerConnect(const char *host, const char *sport)
{
	if(connected == false)
	{
		QString Host(host);
		QString Port(sport);
		emit sigServerStatus(i18n("Connecting to server..."));
		connection->connectToHost(Host, Port.toInt());
	}
}

void KMasqClient::serverDisconnect()
{
	connection->close();
}

/** Dial the given connection */
void KMasqClient::Dial(const char *cname)
{
	if(connected == true)
	{
		if(dialed_out == false)
		{
		  QString cName;
		  cName = cname;
			connection->writeBlock("DIAL:" + cName + "\n", cName.length() + 6);
			status = "dialing";
			busy = true;
		}
	}
}	

/** Kill the current connection */
void KMasqClient::Kill()
{
	if(connected == true)
	{
		status = "hangingup";
		connection->writeBlock("KILL\n", 5);
	}
}

/** Logs off the client from the server
 */
void KMasqClient::sendQuit()
{
	if(connected == true)
	{
		connection->writeBlock("QUIT\n", 5);
	}
}

/**  */
void KMasqClient::List()
{
	if(connected == true)
	{
		status = "listing";
		connection->writeBlock("LIST\n", 5);
	}
}

/** Send auth name and password and make sure its valid
	*/
void KMasqClient::getAuth(const char *user, const char *pass)
{
	if(connected == true)
	{
		QString User;
		User = user;
		connection->writeBlock("USER:" + User + "\n", 6 + User.length());
		QString Pass;
		Pass = pass;
		connection->writeBlock("PASS:" + Pass + "\n", 6 + Pass.length());
	}
}

/** Reads the status from the server
	*/
void KMasqClient::readStatus()
{

}

/** Reads the time of the connection being established from the server
  */
void KMasqClient::readConnectTime()
{
	if(connected == true)
	{
		status = "connecttime";
		connection->writeBlock("TIME\n", 5);
	}
}

/** Reads the current time from the server
	*/
void KMasqClient::readServerTime()
{
	if(connected == true)
	{
		status = "servertime";
		connection->writeBlock("CTIME\n", 6);
	}
}

/** Reads the number of clients connected to the server
 */
void KMasqClient::readCCount()
{
	if(connected == true)
	{
		status = "ccounting";
		connection->writeBlock("CCOUNT\n", 7);
	}
}

/** Reads what the server thinks your IP address is.
	*/
void KMasqClient::readCIP()
{
	if(connected == true)
	{
		status = "ciping";
		connection->writeBlock("CIP\n", 4);
	}
}

/**  */
void KMasqClient::readCInfo(const char *cname)
{
	if(connected == true)
	{
		status = "cinfo";
		QString cName;
		cName = cname;
		connection->writeBlock("CINFO:" + cName + "\n", 7 + cName.length());
	}
}


/**  */
void KMasqClient::readWho()
{
	if(connected == true)
	{
		status = "whoing";
		connection->writeBlock("WHO\n", 4);
	}
}


/**  */
void KMasqClient::readAuthor()
{
	if(connected == true)
	{
		status = "authoring";
		connection->writeBlock("AUTHOR\n", 7);
	}
}

/**  */
void KMasqClient::readVersion()
{
	if(connected == true)
	{
		status = "versioning";
		connection->writeBlock("VERSION\n", 8);
	}
}

/**  */
void KMasqClient::readNetLoad()
{
	if(connected == true)
	{
		status = "netload";
		connection->writeBlock("NETLOAD\n", 8);
	}
}


/** Check if client is connected to server */
bool KMasqClient::isConnected()
{
	return connected;
}

/**  */
bool KMasqClient::isDialed()
{
	if(connected == true && status.left(7) != "dialing" && status.left(12) != "begundialing" && status.left(12) != "begunlicense" && status.left(12) != "begunhistory" && status.left(6) != "ciping" && status.left(6) != "whoing" && status.left(9) != "authoring" && status.left(6) != "ccounting")
	{
		connection->writeBlock("STAT\n", 5);
		status = "stat";
	}
	return dialed_out;
}

bool KMasqClient::isBusy()
{
	return busy;
}

/**  */
void KMasqClient::readHistory()
{
	if(connected == true)
	{
		status = "history";
		connection->writeBlock("HISTORY\n", 8);
	}
}

void KMasqClient::readLicense()
{
	if(connected == true)
	{
		status = "license";
		connection->writeBlock("LICENSE\n", 8);
	}
}

void KMasqClient::readLinfo()
{
	if(connected == true)
	{
		status = "linfo";
		connection->writeBlock("LINFO\n", 6);
	}
}

void KMasqClient::lockDial(bool lock)
{
	if(connected == true)
	{
		if(lock == true)
			connection->writeBlock("LOCK:DIAL\n", 10);
		else
			connection->writeBlock("UNLOCK:DIAL\n", 12);
	}
}

void KMasqClient::lockKill(bool lock)
{
	if(connected == true)
	{
		if(lock == true)
			connection->writeBlock("LOCK:KILL\n", 10);
		else
			connection->writeBlock("UNLOCK:KILL\n", 12);
	}
}

void KMasqClient::slotHostFound()
{
	sigServerStatus(i18n("Server found..."));
}

void KMasqClient::slotConnected()
{
	connected = true;
	sigServerStatus(i18n("Connected to masq server."));
	List();
}

void KMasqClient::slotConnectionClosed()
{
	connected = false;
	sigServerStatus(i18n("Connection to masq server closed."));
}

void KMasqClient::slotReadyRead()
{
	QString line;
	
	while(connection->canReadLine())
	{
		line = connection->readLine();
		if(status.left(5) == "linfo" && line.left(5) != "READY" && line.left(3) != "UP:")
		{
			status = "";
			emit sigSendInfo("linfo", line.left(line.length() - 1));
		}
		
		if(line.left(5) == "READY")
		{
			if(status.left(9) == "hangingup")
				emit sigServerStatus("Disconnected.");
		}
		if(line.left(5) == "BEGIN")
		{
			if(status.left(7) == "dialing")
				status = "begundialing";
			if(status.left(7) == "listing")
				status = "begunlisting";
			if(status.left(7) == "history")
				status = "begunhistory";
			if(status.left(7) == "license")
				status = "begunlicense";
			if(status.left(5) == "cinfo")
				status = "beguncinfo";
			if(status.left(7) == "netload")
				status = "begunnetload";
		}
		
		if(line.left(3) == "END")
		{
			if(status.left(12) == "begunhistory")
			{
				emit sigSendInfo("history", history);
				history = "";
			}
			if(status.left(12) == "begunlicense")
			{
				emit sigSendInfo("license", license);
				license = "";
			}
			if(status.left(10) == "beguncinfo")
			{
				emit sigSendInfo("cinfo", cinfo.left(cinfo.length() - 1));
				cinfo = "";
			}
			if(status.left(12) == "begunnetload")
			{
				emit sigSendInfo("netload", netload.left(cinfo.length() - 1));
				netload = "";
			}
			if(status.left(12) == "begunlisting")
			{
				emit sigSendInfo("connected", "");
			}
			if(status.left(12) == "begundialing")
			{
				busy = false;
			}
			if(status.left(5) != "cinfo")
				status = "";
		}
		
		if(line.left(4) == "DOWN")
		{
			dialed_out = false;
			busy = false;
			emit sigServerStatus(i18n("Not online."));
			status = "";
			readLinfo();
		}
		
		if(line.left(2) == "UP")
		{
//			if(line.mid(line.find(":", line.find(":", 4) - 3) + 1, line.length() - line.find(":", line.find(":", 4) - 3) -2) != "0")
//			{
				if(line.mid(3, line.find(":", 4) - 3).left(18) != "Connection Pending")
				{
					emit sigServerStatus(i18n("Connected to ") + line.mid(3, line.find(":", 4) - 3) + i18n(" at ") + line.mid(line.find(":", line.find(":", 4) - 3) + 1, line.length() - line.find(":", line.find(":", 4) - 3) -2) + " bps");
					emit sigSendInfo("speed", line.mid(line.find(":", line.find(":", 4) - 3) + 1, line.length() - line.find(":", line.find(":", 4) - 3) -2));
					dialed_out = true;
					busy = false;
				}
				else
				{
					emit sigServerStatus(i18n("A connection is being dialed..."));
					busy = true;
				}
//			}
//			else
//				emit sigServerStatus(i18n("A connection is being dialed..."));
			if(status.left(7) == "dialing")
				emit sigServerStatus(i18n("Already dialed!"));
		}
		
		if(line.left(5) == "ERROR")
		{
			emit sigSendInfo("error", i18n(line.mid(6, line.length() - 7)));
			status = "";
			busy = false;
		}
		
		if(line.left(6) == "LOCKED")
		{
			emit sigServerStatus(i18n("The connection has been locked."));
		}
		
		if(line.left(8) == "UNLOCKED")
		{
			emit sigServerStatus(i18n("The connection has been unlocked."));
		}
		
		if(status.left(12) == "begunlisting" && line.left(5) != "BEGIN" && line.left(3) != "END" && line.left(5) != "READY" && line.left(3) != "UP:")
			emit sigNewConnection(line.left(line.length() - 1));

		if(status.left(10) == "beguncinfo" && line.left(5) != "BEGIN" && line.left(3) != "END" && line.left(5) != "READY" && line.left(3) != "UP:")
			cinfo = cinfo + line;
			
		if(status.left(12) == "begundialing" && line.left(5) != "BEGIN" && line.left(3) != "END" && line.left(5) != "READY" && line.left(3) != "UP:" && line.left(6) != "ERROR:")
			emit sigServerStatus(i18n(line.left(line.length() - 1)));
			
		if(status.left(12) == "begunhistory" && line.left(5) != "BEGIN" && line.left(3) != "END" && line.left(5) != "READY" && line.left(3) != "UP:")
			history = history + line;
			
		if(status.left(12) == "begunlicense" && line.left(5) != "BEGIN" && line.left(3) != "END" && line.left(5) != "READY" && line.left(3) != "UP:")
			license = license + line;
			


		if(status.left(10) == "servertime" && line.left(5) != "READY" && line.left(3) != "UP:")
		{
			status = "";
			emit sigSendInfo("servertime", line.left(line.length() - 1));
		}
		
		if(status.left(11) == "connecttime" && line.left(5) != "READY" && line.left(3) != "UP:")
		{
			status = "";
			emit sigSendInfo("connecttime", line.left(line.length() - 1));
		}
		
		if(status.left(12) == "begunnetload" && line.left(5) != "BEGIN" && line.left(3) != "END" && line.left(5) != "READY" && line.left(3) != "UP:")
			netload = netload + line;
		
		if(status.left(9) == "authoring" && line.left(5) != "READY" && line.left(3) != "UP:")
		{
			status = "";
			emit sigSendInfo("author", line.left(line.length() - 1));
		}
		
		if(status.left(10) == "versioning" && line.left(5) != "READY" && line.left(3) != "UP:")
		{
			status = "";
			emit sigSendInfo("version", line.left(line.length() - 1));
		}
		
		if(status.left(6) == "whoing" && line.left(5) != "READY" && line.left(3) != "UP:")
		{
			status = "";
			emit sigSendInfo("who", line.left(line.length() - 1));
		}
		
		if(status.left(9) == "ccounting" && line.left(5) != "READY" && line.left(3) != "UP:")
		{
			status = "";
			emit sigSendInfo("ccount", line.left(line.length() - 1));
		}
		if(status.left(6) == "ciping" && line.left(5) != "READY" && line.left(3) != "UP:")
		{
			status = "";
			emit sigSendInfo("cip", line.left(line.length() - 1));
		}
	}
}

void KMasqClient::slotError(int error)
{
	//Called when a socket error occurs
	if(error == QSocket::ErrConnectionRefused)
		sigServerStatus(i18n("Connection to masq server refused."));
	if(error == QSocket::ErrHostNotFound)
		sigServerStatus(i18n("Masq server not found."));
	if(error == QSocket::ErrSocketRead)
		sigServerStatus(i18n("Error reading from server."));
}
