/***************************************************************************
                          kmasqclient.h  -  description
                             -------------------
    begin                : Sun Mar 25 2001
    copyright            : (C) 2001 by Stephan de Georgio
    email                : kmasqdialer@stephan.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMASQCLIENT_H
#define KMASQCLIENT_H


/**
  *@author Stephan de Georgio
  */

#include <qobject.h>
#include <qstring.h>

#include <qsocket.h>

/**Class for connecting to server
  *@author Darren Poulson
  *@short All methods for communicating with the server are here
  */

class KMasqClient : public QObject
{
	Q_OBJECT
public:
	/** Constructor */
	KMasqClient(QObject *parent=0, const char *name=0);

	/** Destructor */
	~KMasqClient();
	
  /** Kill the current connection
	* @param none
	*/
  void Kill();

  /** Dial the given connection
  * @param cname Connection name to dial
  */
  void Dial(const char *);

  /** Connect to the modem server
	* @param host Hostname of server
	* @param	port Port number of server
	*/
	void ServerConnect(const char *, const char *);

	void serverDisconnect();
  /** Check if client is connected to server
	* @param none
	* @return True or false, is the client connected to a server
	*/
  bool isConnected();

  /** Reads the time of the connection being established from the server
	* @param none
  */
  void readConnectTime();

  /** Reads the status from the server
	* @param none
	*/
  void readStatus();

  /** Reads what the server thinks your IP address is.
	* @param none
	*/
	void readCIP();

  /** Reads the number of clients connected to the server
	* @param none
	*/
  void readCCount();

  /** Reads the current time from the server
	* @param none
	*/
  void readServerTime();

  /** Send auth name and password and make sure its valid
	* @param user Username
	* @param pass Password
	*/
  void getAuth(const char *, const char *);
  /** Logs off the client from the server
	* @param none
	* @return none
  */
  void sendQuit();

  /** Reads the connection information
	* @param cname Connection name to get info for
	* @return QString containing all the data
	*/
  void readCInfo(const char *);

  /**  */
  bool isDialed();

  bool isBusy();

  /**  */
  void readWho();

  /**  */
  void readVersion();

  /**  */
  void readAuthor();

  /**  */
  void readNetLoad();
  /**  */
  void readHistory();
  /**  */
  void readLicense();
	/**  */
  void readLinfo();
	/** locks a connection from dialing */
	void lockDial(bool);
	/** locks a connection from killin */
	void lockKill(bool);


private: // Private attributes

  /** Is there a server connection */
  bool connected;

  /** is there an active connection? */
	bool dialed_out;

  /** List all available connections */
  void List();

  QString status;

  QString history;

  QString license;

  QString cinfo;

  QString netload;

  bool busy;

signals: // Signals

  /** A new connection option from server */
  void sigNewConnection(const char *);

	/** Sends server status messages */
	void sigServerStatus(const char *);
	
	void sigSendInfo(QString, QString);

private slots:
	void slotHostFound();
	void slotConnected();
	void slotConnectionClosed();
	void slotReadyRead();
	void slotError(int);
	
private:
	QSocket *connection;

};
#endif
