/***************************************************************************
                          kmasqconfig.cpp  -  description
                             -------------------
    begin                : Sat Mar 24 2001
    copyright            : (C) 2001 by Stephan de Georgio
    email                : kmasqdialer@stephan.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmasqconfig.h"
#include <klocale.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>

KMasqConfig::KMasqConfig(KMasqDialer *parent, const char *name ) : QDialog(parent, name, true)
{
	config = parent->returnConfig();
	config->setGroup("Options");
	setFixedSize(215, 205);
	setCaption(i18n(i18n("Configure - KMasqDialer")));
	
	QGroupBox *frame = new QGroupBox(i18n("Client Options"), this);
	frame->setGeometry(5, 5, 205, 165);
	
	servernamelabel = new QLabel(i18n("Server"), frame);
	servernamelabel->setGeometry(7, 15, 40, 15);
	servername = new QLineEdit(frame);
	servername->setGeometry(5, 30, 150, 25);
	QWhatsThis::add(servername, i18n("The name/ip address of the server that this client will connect to."));
	
	serverportlabel = new QLabel(i18n("Port"), frame);
	serverportlabel->setGeometry(160, 15, 40, 15);
	serverport = new QLineEdit(frame);
	serverport->setGeometry(158, 30, 40, 25);
	QWhatsThis::add(serverport, i18n("The port on the server that this client will to connect to."));
	
	autoconnect = new QCheckBox(i18n("Autoconnect to server"), frame);
	autoconnect->setGeometry(5, 60, 190, 15);
	QWhatsThis::add(autoconnect, i18n("Do you want this client to automatically connect to the server?"));
	
	autoauth = new QCheckBox(i18n("Authorize user on startup"), frame);
	autoauth->setGeometry(5, 80, 190, 15);
	QWhatsThis::add(autoauth, i18n("Do you want this client to automatically authorise you on startup?"));
	
	autodial = new QCheckBox(i18n("Automatically dial connection"), frame);
	autodial->setGeometry(5, 100, 190, 15);
	QWhatsThis::add(autodial, i18n("Do you want this client to automatically dial to a connection on statup?"));
	
	autodialnamelabel = new QLabel(i18n("Dial which connection"), frame);
	autodialnamelabel->setGeometry(5, 115, 150, 15);
	autodialname = new QLineEdit(frame);
	autodialname->setGeometry(5, 130, 193, 25);
	QWhatsThis::add(autodialname, i18n("Enter here the name of the connection that you want to automatically dial.  It must be listed in the connections combo box in the main window."));
	if(!autodial->isChecked())
		autodialname->setEnabled(false);
	connect(autodial, SIGNAL(toggled(bool)), this, SLOT(slotAutoDialClicked(bool)));
	
	ok = new QPushButton(i18n("&OK"), this);
	ok->setDefault(true);
	connect(ok, SIGNAL(clicked()), this, SLOT(slotOk()));
	ok->setGeometry( 130, 175, 80, 25);
	cancel = new QPushButton(i18n("&Cancel"), this);
	connect(cancel, SIGNAL(clicked()), this, SLOT(slotCancel()));
	cancel->setGeometry( 45, 175, 80, 25);

	servername->setText(config->readEntry("servername"));
	serverport->setText(config->readEntry("serverport", "222"));
	autoconnect->setChecked(config->readBoolEntry("autoconnect"));
	autoauth->setChecked(config->readBoolEntry("autoauth"));
	autodial->setChecked(config->readBoolEntry("autodial"));
	autodialname->setText(config->readEntry("autodialname"));
}

KMasqConfig::~KMasqConfig()
{
}

void KMasqConfig::saveConfig()
{
	config->setGroup("Options");
	config->writeEntry("serverport",serverport->text());
	config->writeEntry("servername",servername->text());
	config->writeEntry("autoconnect",autoconnect->isChecked());
	config->writeEntry("autoauth", autoauth->isChecked());
	config->writeEntry("autodial", autodial->isChecked());
	config->writeEntry("autodialname", autodialname->text());
}

void KMasqConfig::slotOk()
{
	saveConfig();
	close(true);
}

void KMasqConfig::slotCancel()
{
	close(true);
}

void KMasqConfig::slotAutoDialClicked(bool b)
{
	autodialname->setEnabled(b);
}


