/***************************************************************************
                          kmasqdialer.cpp  -  description
                             -------------------
    begin                : Thu Mar 22 16:20:16 GMT 2001
    copyright            : (C) 2001 by Stephan de Georgio
    email                : kmasqdialer@stephan.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmasqclient.h"
#include "kmasqconnectioninfo.h"
#include "kmasqserverinfo.h"
#include "kmasqlicense.h"
#include "kmasqauth.h"
#include "kmasqconfig.h"
#include "kmasqdialer.h"
#include "kmasqdock.h"

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include <kstatusbar.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <qpainter.h>
#include <qwhatsthis.h>

KMasqDialer::KMasqDialer(QWidget *parent, const char *name) :  KMainWindow(parent, name)
{
	setCaption(i18n("KMasqDialer"));
	///Create a new server client
	server = new KMasqClient;
	connect(server, SIGNAL(sigNewConnection(const char *)), this, SLOT(slotAddConnection(const char *)));
	connect(server, SIGNAL(sigSendInfo(QString, QString)), this, SLOT(slotGotInfo(QString, QString)));
	//Create a timer to poll the server
	pollTimer = new QTimer;
	connect(pollTimer, SIGNAL(timeout()), this, SLOT(slotUpdateStats()));
	pollTimer->start(1000);
	//initialise stuff
	initWindow();
	initMenu();
	initStatusBar();
	initDock();
	
	//Auto connect to the server if set in config file
	kapp->config()->setGroup("Options");
	if(kapp->config()->readBoolEntry("autoconnect", false))
		slotServerConnect();
}

KMasqDialer::~KMasqDialer()
{
}

void KMasqDialer::initWindow()
{
	//Initialise main widgets
	setFixedSize(270, 170);
	connections = new KComboBox(this);
	connections->setGeometry(5, 35, 185, 25);
	QWhatsThis::add(connections, i18n("A list of connections that can be dialed."));
	dial = new KPushButton(i18n("&Dial"), this);
	dial->setEnabled(false);
	dial->setGeometry(23, 70, 70, 25);
	QWhatsThis::add(dial, i18n("Dials the selected connection."));
	hangUp = new KPushButton(i18n("Hang &Up"), this);
	hangUp->setGeometry(102, 70, 70, 25);
	hangUp->setEnabled(false);
	QWhatsThis::add(hangUp, i18n("Hangs up the currently dialed connection."));
	connect(dial, SIGNAL(clicked()), this, SLOT(slotConnectionDial()));
	connect(hangUp, SIGNAL(clicked()), this, SLOT(slotConnectionHangup()));
	
	connectedTime = new QLabel(i18n("Not online."), this);
	connectedTime->setGeometry(10, 100, 180, 15);
	connectedTime->setAlignment(AlignCenter);
	
	//Initalise output graph
	QLabel *lblOutput = new QLabel(i18n("Output"), this);
	lblOutput->setGeometry(213, 30, 65, 15);
	frameOutput = new QFrame(this);
	frameOutput->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	frameOutput->setGeometry(200, 46, 65, 40);
	frameOutput->setBackgroundColor(Qt::black);
	QWhatsThis::add(frameOutput, i18n("A graph of the outgoing network activity."));
	
	//Initialise input graph
	QLabel *lblInput = new QLabel(i18n("Input"), this);
	lblInput->setGeometry(220, 88, 65, 15);
	frameInput = new QFrame(this);
	frameInput->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	frameInput->setGeometry(200, 104, 65, 40);
	frameInput->setBackgroundColor(Qt::black);
	QWhatsThis::add(frameInput, i18n("A graph of the incoming network activity."));
	
	//Set all graph values to 0
	for(int x = 1; x<63; x++)
	{
		intInput[x] = 0;
		intOutput[x] = 0;
	}
	bytesIn = 0;
	lastBytesIn = 0;
	bytesOut = 0;
	lastBytesOut = 0;
	speed = 0;

	//Initialise input and output netload labels
	netLoadIn = new QLabel(this);
	netLoadIn->setGeometry(10, 130, 180, 15);
	netLoadIn->setAlignment(AlignCenter);
	netLoadOut = new QLabel(this);
	netLoadOut->setGeometry(10, 115, 180, 15);
	netLoadOut->setAlignment(AlignCenter);
	
	//Set the position of the window on the screen
	kapp->config()->setGroup("Options");
	setGeometry(kapp->config()->readNumEntry("left", 0), kapp->config()->readNumEntry("top", 0), 245, 170);
	/*
edit=new QLineEdit(this);
edit->setGeometry(2, 100, 100, 25);*/
}

void KMasqDialer::initMenu()
{
	//Initialise the menus
	serverMenu = new QPopupMenu;
	serverMenu->insertItem(BarIcon("connect_established"), i18n("Connect"), 1);
	serverMenu->connectItem(1, this, SLOT(slotServerConnect()));
	serverMenu->insertItem(BarIcon("connect_no"), i18n("Disconnect"), 2);
	serverMenu->setItemEnabled(2, false);
	serverMenu->connectItem(2, this, SLOT(slotServerDisconnect()));
	serverMenu->insertSeparator();
	serverMenu->insertItem(i18n("Change User..."), 6);
	serverMenu->connectItem(6, this, SLOT(slotServerChangeUser()));
	serverMenu->insertSeparator();
	serverMenu->insertItem(i18n("Info..."), 3);
	serverMenu->connectItem(3, this, SLOT(slotServerInfo()));
	serverMenu->insertItem(i18n("History..."), 4);
	serverMenu->connectItem(4, this, SLOT(slotServerHistory()));
	serverMenu->insertItem(i18n("License..."), 5);
	serverMenu->connectItem(5, this, SLOT(slotServerLicense()));
	serverMenu->insertSeparator();
	serverMenu->insertItem(BarIcon("configure"), i18n("Configure..."), this, SLOT(slotServerConfigure()));
	serverMenu->insertSeparator();
	serverMenu->insertItem(BarIcon("exit"), i18n("Quit"), this, SLOT(slotServerQuit()));
	
	connectionMenu = new QPopupMenu;
	connectionMenu->insertItem(i18n("&Dial"), 8);
	connectionMenu->setItemEnabled(8, false);
	connectionMenu->connectItem(8, this, SLOT(slotConnectionDial()));
	connectionMenu->insertItem(i18n("&Hangup"), 9);
	connectionMenu->setItemEnabled(9, false);
	connectionMenu->connectItem(9, this, SLOT(slotConnectionHangup()));	
	connectionMenu->insertSeparator();
	
	connectionMenu->insertItem(i18n("&Lock from dialing"), 11);
	connectionMenu->connectItem(11, this, SLOT(slotConnectionLockDial()));
	connectionMenu->insertItem(i18n("&Lock from hanging up"), 12);
	connectionMenu->connectItem(12, this, SLOT(slotConnectionLockKill()));	
	connectionMenu->insertSeparator();
	
	connectionMenu->insertItem(i18n("&Info..."), 10);
	connectionMenu->connectItem(10, this, SLOT(slotConnectionInfo()));
	
	helpMenu_ = new QPopupMenu;
	helpMenu_ = helpMenu("KMasqDialer 2.05\n\n(c) 2001 by\nStephan de Georgio\nkmasqdialer@stephan.co.uk");
	
	menuBar()->insertItem(i18n("&Server"), serverMenu);
	menuBar()->insertItem(i18n("&Connection"), connectionMenu);
	menuBar()->insertItem(i18n("&Help"), helpMenu_);
}

void KMasqDialer::initStatusBar()
{
	//Initialise the status bar
	status = new QLabel(statusBar());
  status->setText(i18n("Welcome to KMasqDialer!"));
	statusBar()->addWidget(status, 1);
	statusBar()->setFixedWidth(387);
	connect(server, SIGNAL(sigServerStatus(const char *)), this, SLOT(slotSetStatusMsg(const char *)));
}

void KMasqDialer::initDock()
{
	//Initialise the dock
	dock = new KMasqDock;
	dock->show();
	connect(dock, SIGNAL(sigChangeStatus(const char *)), this, SLOT(slotSetStatus(const char *)));
	connect(this, SIGNAL(setDockStatus(const char *)), dock, SLOT(slotSetStatus(const char *)));
	connect(this, SIGNAL(sigSetToolTip(const char *)), dock, SLOT(slotSetToolTip(const char *)));
	kapp->config()->setGroup("Options");
	if(kapp->config()->readBoolEntry("visible", true))
		show();
	else
	{
		hide();
		emit setDockStatus("minimized");
	}
	connect(this, SIGNAL(sigUpdateDock(float, float, float, float)), dock, SLOT(slotUpdateDock(float, float, float, float)));
}

KConfig *KMasqDialer::returnConfig()
{
	return kapp->config();
}

void KMasqDialer::slotServerConnect()
{
	kapp->config()->setGroup("Options");
	//Connect to the server
	server->ServerConnect(kapp->config()->readEntry("servername", ""), kapp->config()->readEntry("serverport", ""));
}

void KMasqDialer::slotServerDisconnect()
{
	status->setText(i18n("Disconnecting from server..."));
	kapp->config()->writeEntry("defaultdial", connections->currentText());
	server->sendQuit();
}

void KMasqDialer::slotServerChangeUser()
{
	KMasqAuth *auth = new KMasqAuth(this);
	connect(auth, SIGNAL(authenticated(QString, QString)), this, SLOT(slotAuthenticated(QString, QString)));
	auth->show();
}

void KMasqDialer::slotServerInfo()
{
	//Create the server info box
	serverInfo = new KMasqServerInfo(this);
	server->readCIP();
}

void KMasqDialer::slotServerHistory()
{
	server->readHistory();
}

void KMasqDialer::slotServerLicense()
{
	server->readLicense();
}

void KMasqDialer::slotServerConfigure()
{
	KMasqConfig *configure = new KMasqConfig(this);
	configure->show();
}

void KMasqDialer::slotServerQuit()
{
	slotServerDisconnect();
	close();
}

void KMasqDialer::slotConnectionDial()
{
  server->Dial(connections->text(connections->currentItem()));
}

void KMasqDialer::slotConnectionHangup()
{
	status->setText(i18n("Hanging up..."));
	server->Kill();
}

void KMasqDialer::slotConnectionInfo()
{
	//Create the connection info box
	connectionInfo = new KMasqConnectionInfo(this);
	cCounter = 0;
	server->readCInfo(connections->text(cCounter));
	connectionInfo->show();
}

void KMasqDialer::resizeEvent(QResizeEvent *)
{
	setFixedSize(270, 170);
}


void KMasqDialer::slotSetStatusMsg(const char *msg)
{
	status->setText(msg);
	emit sigSetToolTip(msg);
}

void KMasqDialer::slotAddConnection(const char *cname)
{
	connections->insertItem(cname, -1);
}

void KMasqDialer::slotAuthenticated(QString username, QString password)
{
	server->getAuth(username, password);
}

void KMasqDialer::slotConnectionLockDial()
{
	if(dialLock == true)
		server->lockDial(false);
	else
		server->lockDial(true);
}

void KMasqDialer::slotConnectionLockKill()
{
	if(killLock == true)
		server->lockKill(false);
	else
		server->lockKill(true);
}

void KMasqDialer::slotSetStatus(const char *status)
{
	//Gets called from the dock to make the main program do stuff
	kapp->config()->setGroup("Options");
	QString temp(status);
	if(temp == "show")
	{
		show();
	}
	if(temp == "hide")
	{
		hide();
	}
	if(temp == "dial")
	{
		slotConnectionDial();
	}
	if(temp == "hangup")
	{
		slotConnectionHangup();
	}
	if(temp == "quit")
	{
		slotServerQuit();
	}
}

void KMasqDialer::slotUpdateStats()
{	
	char connected[1024];
	int contime;
	QString contimesec;
	QString contimemin;
	QString contimehr;
	contime = (serverTime - connectTime);
	contimesec.setNum((int)fmod(contime,60));
	contimehr.setNum((int)abs(contime/3600));
	contimemin.setNum((int)abs((contime-(contimehr.toInt()*3600))/60));
	
	if(contimesec.length() != 2)
		contimesec.insert(0, "0");
	if(contimehr.length() != 2)
		contimehr.insert(0, "0");
	if(contimemin.length() != 2)
		contimemin.insert(0, "0");
	
	QString temp;
	temp = connected;
	
	//Checks if we are connected to the server
	if(server->isConnected())
	{
		emit setDockStatus("connected");
		if(server->isDialed())
		{
			if(!server->isBusy())
			{
				emit setDockStatus("connected");
				emit setDockStatus("dialed");
				serverMenu->setItemEnabled(1, false);
				serverMenu->setItemEnabled(2, true);
				serverMenu->setItemEnabled(3, true);
				serverMenu->setItemEnabled(4, true);
				serverMenu->setItemEnabled(5, true);
				serverMenu->setItemEnabled(6, true);
				connectionMenu->setItemEnabled(8, false);
				connectionMenu->setItemEnabled(9, true);
				connectionMenu->setItemEnabled(10, true);
				connectionMenu->setItemEnabled(11, true);
				connectionMenu->setItemEnabled(12, true);
				hangUp->setEnabled(true);
				dial->setEnabled(false);
				server->readConnectTime();
				connectedTime->setText(i18n("Connected - ") + contimehr + ":" + contimemin + ":" + contimesec);
			}
			else
			{
				emit setDockStatus("hungup");
				emit setDockStatus("dialing");
				emit setDockStatus("connected");
				serverMenu->setItemEnabled(1, false);
				serverMenu->setItemEnabled(2, true);
				serverMenu->setItemEnabled(3, true);
				serverMenu->setItemEnabled(4, true);
				serverMenu->setItemEnabled(5, true);
				serverMenu->setItemEnabled(6, true);
				connectionMenu->setItemEnabled(8, false);
				connectionMenu->setItemEnabled(9, false);
				connectionMenu->setItemEnabled(10, true);
				connectionMenu->setItemEnabled(11, true);
				connectionMenu->setItemEnabled(12, true);
				hangUp->setEnabled(false);
				dial->setEnabled(false);
				connectedTime->setText(i18n("Not online."));
				netLoadIn->setText("");
				netLoadOut->setText("");
			}
		}
		else
		{
			if(server->isBusy())
			{
				emit setDockStatus("hungup");
				emit setDockStatus("dialing");
				emit setDockStatus("disconnected");
				hangUp->setEnabled(false);
				dial->setEnabled(false);
				serverMenu->setItemEnabled(1, false);
				serverMenu->setItemEnabled(2, false);
				serverMenu->setItemEnabled(3, false);
				serverMenu->setItemEnabled(4, false);
				serverMenu->setItemEnabled(5, false);
				serverMenu->setItemEnabled(6, false);
				connectionMenu->setItemEnabled(8, false);
				connectionMenu->setItemEnabled(9, false);
				connectionMenu->setItemEnabled(10, false);
				connectionMenu->setItemEnabled(11, false);
				connectionMenu->setItemEnabled(12, false);
			}
			else
			{
				if(connections->count() > 0)
				{
					emit setDockStatus("hungup");
					emit setDockStatus("connected");
					serverMenu->setItemEnabled(1, false);
					serverMenu->setItemEnabled(2, true);
					serverMenu->setItemEnabled(3, true);
					serverMenu->setItemEnabled(4, true);
					serverMenu->setItemEnabled(5, true);
					serverMenu->setItemEnabled(6, true);
					connectionMenu->setItemEnabled(8, true);
					connectionMenu->setItemEnabled(9, false);
					connectionMenu->setItemEnabled(10, true);
					connectionMenu->setItemEnabled(11, true);
					connectionMenu->setItemEnabled(12, true);
					hangUp->setEnabled(false);
					dial->setEnabled(true);
					connectedTime->setText(i18n("Not online."));
					netLoadIn->setText("");
					netLoadOut->setText("");
				}
			}
		}
		
	}
	else
	{
		emit setDockStatus("hungup");
		emit setDockStatus("disconnected");
		serverMenu->setItemEnabled(1, true);
		serverMenu->setItemEnabled(2, false);
		serverMenu->setItemEnabled(3, false);
		serverMenu->setItemEnabled(4, false);
		serverMenu->setItemEnabled(5, false);
		serverMenu->setItemEnabled(6, false);
		connectionMenu->setItemEnabled(8, false);
		connectionMenu->setItemEnabled(9, false);
		connectionMenu->setItemEnabled(10, false);
		connectionMenu->setItemEnabled(11, false);
		connectionMenu->setItemEnabled(12, false);
		hangUp->setEnabled(false);
		dial->setEnabled(false);
		connections->clear();
		connectedTime->setText(i18n("Not online."));
		netLoadIn->setText("");
		netLoadOut->setText("");
	}
	
	//Checks if this window has been minimized
	if(isMinimized())
	{
		//Hide the window
		hide();
		//Tell the dock that this window has been minimized
		emit setDockStatus("minimized");
	}
/*			for(int x = 1; x<62; x++)
		{
			intInput[x] = intInput[x+1];
			intOutput[x] = intOutput[x+1];
		}
	intOutput[62] = edit->text().toInt();
	update();*/
}

void KMasqDialer::slotGotInfo(QString type, QString info)
{
	if(type.left(7) == "history")
	{
		//Load the history box
		KMasqLicense *history = new KMasqLicense(info, false, this);
		history->show();
	}
	if(type.left(7) == "license")
	{
		//Load the license box
		KMasqLicense *license = new KMasqLicense(info, true, this);
		license->show();
	}
	if(type.left(3) == "cip")
	{
		serverInfo->setIPText(info);
		server->readCCount();
	}
	if(type.left(6) == "ccount")
	{
		serverInfo->setCountNumber(info.toInt());
		server->readWho();
	}
	if(type.left(3) == "who")
	{
		serverInfo->setClientNames(info);
		server->readVersion();
	}
	if(type.left(7) == "version")
	{
		serverInfo->setVersion(info);
		server->readAuthor();
	}
	if(type.left(6) == "author")
	{
		serverInfo->setAuthor(info);
		serverInfo->show();
	}
	
	if(type.left(5) == "cinfo")
	{
		connectionInfo->addInfoTab(info);
		cCounter++;
		if(cCounter < connections->count())
			server->readCInfo(connections->text(cCounter));
	}
	
	if(type.left(10) == "servertime")
	{
		serverTime = info.toInt();
		server->readNetLoad();
	}
	
	if(type.left(11) == "connecttime")
	{
		connectTime = info.toInt();
		server->readServerTime();
	}
	
	if(type.left(5) == "error")
	{
		//Pops up a message box containing the error message sent from the server
		KMessageBox::sorry(this, info);
		//Stops these buttons getting stuck down when the message box come up
		dial->setDown(false);
		hangUp->setDown(false);
	}
	
	if(type.left(10) == "netload")
	{
		//Calculated the current net load
		lastBytesIn = bytesIn;
		lastBytesOut = bytesOut;
		sscanf(info, "INPUT: %f %f OUTPUT: %f %f", &bytesIn, &packetsIn, &bytesOut, &packetsOut);

		QString units;
		float value;
		float bytesSec;
		char temp[1024];
		
		if(bytesIn < 1024*100)
		{
			units = "";
			value = bytesIn;
		}
		else if(bytesIn < 1024*1024*100)	
		{
			units = "K";
			value = bytesIn/1024;
		}
		else
		{
			units = "M";
			value = bytesIn/1024/1024;
		}
		bytesSec = bytesIn - lastBytesIn;
		bytesSec /= 1024;
		snprintf(temp, 1023, "%.0f%sb recieved (%.2fKb/sec)", value, units.operator const char *(), bytesSec);
		QString toolTipTemp(temp);
		netLoadIn->setText(temp);
		
		
		if(bytesOut < 1024*100)
		{
			units = "";
			value = bytesOut;
		}
		else if(bytesOut < 1024*1024*100)	
		{
			units = "K";
			value = bytesOut/1024;
		}
		else
		{
			units = "M";
			value = bytesOut/1024/1024;
		}
		bytesSec = bytesOut - lastBytesOut;
		bytesSec /= 1024;
		snprintf(temp, 1023, "%.0f%sb sent (%.2fKb/sec)", value, units.operator const char *(), bytesSec);
		netLoadOut->setText(temp);
		toolTipTemp += ", ";
		toolTipTemp +=temp;
		emit sigSetToolTip(toolTipTemp);

		//Set the graphs
		for(int x = 1; x<62; x++)
		{
			intInput[x] = intInput[x+1];
			intOutput[x] = intOutput[x+1];
		}
		if(speed != 0)
		{
		speed /= 8;
		speed /= 1024;
		intInput[62] = ((bytesIn - lastBytesIn)/1024/speed * 36.0);
		intOutput[62] = ((bytesOut - lastBytesOut)/1024/speed * 36.0);
		}
		else
		{
		intInput[62] = ((bytesIn - lastBytesIn)/1024/6.9 * 36.0);
		intOutput[62] = ((bytesOut - lastBytesOut)/1024/6.9 * 36.0);
		}
		for(int x = 1; x<63; x++)
		{
			if(intInput[x] > 36)
				intInput[x] = 36;
			if(intOutput[x] > 36)
				intOutput[x] = 36;
			if(intInput[x] < 0)
				intInput[x] = 0;
			if(intOutput[x] < 0)
				intOutput[x] = 0;
		}
		
		//Paints the graphs
		QPixmap *pixOutput = new QPixmap;
		pixOutput->resize(62, 36);
		QPainter *paintOutput = new QPainter(pixOutput);
		pixOutput->fill(black);
		paintOutput->setBackgroundColor(black);
		paintOutput->eraseRect(0, 0, 62, 36);
		QPen pen(cyan, 1);
		paintOutput->setPen(pen);
		for(int x = 1; x<63; x++)
		{
			if(intOutput[x] != 0)
				paintOutput->drawLine(x, 36 - (int)intOutput[x], x, 36);
		}
		paintOutput->flush();
		bitBlt(frameOutput, 1, 2, pixOutput, 0, 0, 62, 36);	
		delete paintOutput;
		delete pixOutput;
		
		QPixmap *pixInput = new QPixmap;
		pixInput->resize(62, 36);
		QPainter *paintInput = new QPainter(pixInput);
		pixInput->fill(black);
		paintInput->setBackgroundColor(black);
		paintInput->eraseRect(0, 0, 62, 36);
		paintInput->setPen(pen);
		for(int x = 1; x<63; x++)
		{
			if(intInput[x] != 0)
				paintInput->drawLine(x, 36 - (int)intInput[x], x, 36);
		}
		paintInput->flush();
		bitBlt(frameInput, 1, 2, pixInput, 0, 0, 62, 36);
		delete paintInput;
		delete pixInput;
	//	update();
		emit sigUpdateDock(bytesIn, lastBytesIn, bytesOut, lastBytesOut);
		server->readLinfo();
	}
	
	if(type.left(5) == "speed")
	{
		speed = info.toInt();
	}
	
	if(type.left(9) == "connected")
	{
		for(int x = 0; x<connections->count(); x++)
		{
			if(connections->text(x) == kapp->config()->readEntry("defaultdial", ""))
				connections->setCurrentItem(x);
		}
		if(kapp->config()->readBoolEntry("autoauth", false) == true)
		{
			if(kapp->config()->readBoolEntry("savepassword", false) == false)
			{
				KMasqAuth *auth = new KMasqAuth(this);
				connect(auth, SIGNAL(authenticated(QString, QString)), this, SLOT(slotAuthenticated(QString, QString)));
				auth->show();
			}
			else
			{
				slotAuthenticated(kapp->config()->readEntry("username", ""), kapp->config()->readEntry("userpassword", ""));
			}
		}

		if(kapp->config()->readBoolEntry("autodial", false) == true)
		{
			server->Dial(kapp->config()->readEntry("autodialname", ""));
		}
	}
	
	if(type.left(5) == "linfo")
	{
		if(info.left(1) == "0")
		{
			connectionMenu->setItemChecked(11, false);
			dialLock = false;
		}
		else
		{
			connectionMenu->setItemChecked(11, true);
			dialLock = true;
		}
		if(info.mid(2,1) == "0")
		{	
			connectionMenu->setItemChecked(12, false);
			killLock = false;
		}
		else
		{
			connectionMenu->setItemChecked(12, true);
			killLock = true;
		}
	}
}

void KMasqDialer::paintEvent(QPaintEvent *)
{
	for(int x = 1; x<63; x++)
	{
		if(intInput[x] > 36)
			intInput[x] = 36;
		if(intOutput[x] > 36)
			intOutput[x] = 36;
		if(intInput[x] < 0)
			intInput[x] = 0;
		if(intOutput[x] < 0)
			intOutput[x] = 0;
	}
	//Paints the graphs
	QPixmap *pixOutput = new QPixmap;
	pixOutput->resize(62, 36);
	QPainter *paintOutput = new QPainter(pixOutput);
	pixOutput->fill(black);
	paintOutput->setBackgroundColor(black);
	paintOutput->eraseRect(0, 0, 62, 36);
	QPen pen(cyan, 1);
	paintOutput->setPen(pen);
	for(int x = 1; x<63; x++)
	{
		if(intOutput[x] != 0)
			paintOutput->drawLine(x, 36 - (int)intOutput[x], x, 36);
	}
	paintOutput->flush();
	bitBlt(frameOutput, 1, 2, pixOutput, 0, 0, 62, 36);	
	delete paintOutput;
	delete pixOutput;
	
	QPixmap *pixInput = new QPixmap;
	pixInput->resize(62, 36);
	QPainter *paintInput = new QPainter(pixInput);
	pixInput->fill(black);
	paintInput->setBackgroundColor(black);
	paintInput->eraseRect(0, 0, 62, 36);
	paintInput->setPen(pen);
	for(int x = 1; x<63; x++)
	{
		if(intInput[x] != 0)
			paintInput->drawLine(x, 36 - (int)intInput[x], x, 36);
	}
	paintInput->flush();
	bitBlt(frameInput, 1, 2, pixInput, 0, 0, 62, 36);
	delete paintInput;
	delete pixInput;
}

bool KMasqDialer::queryClose()
{
	if(server->isConnected())
		slotServerDisconnect();
	kapp->config()->setGroup("Options");
	kapp->config()->writeEntry("left", x());
	kapp->config()->writeEntry("top", y());
	kapp->config()->writeEntry("visible", isVisible());
	return true;
}

