/***************************************************************************
                          kmasqdialer.h  -  description
                             -------------------
    begin                : Thu Mar 22 16:20:16 GMT 2001
    copyright            : (C) 2001 by Stephan de Georgio
    email                : kmasqdialer@stephan.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMASQDIALER_H
#define KMASQDIALER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <kmainwindow.h>
#include <kcombobox.h>
#include <kpushbutton.h>

#include <qpopupmenu.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qlineedit.h>
class KMasqClient;
class KMasqDock;
class KMasqServerInfo;
class KMasqConnectionInfo;

/** KMasqDialer is the base class of the porject */
class KMasqDialer : public KMainWindow
{
Q_OBJECT
public:
	/** construtor */
	KMasqDialer(QWidget* parent=0, const char *name=0);
	/** destructor */
	~KMasqDialer();
	KConfig *returnConfig();

protected:
	void resizeEvent(QResizeEvent *);
	void paintEvent(QPaintEvent *);
	bool queryClose();
	
private:
	void initWindow();
	void initMenu();
	void initStatusBar();
	void initDock();
	KComboBox *connections;
	KPushButton *dial;
	KPushButton *hangUp;
	KMasqClient *server;
	QPopupMenu *serverMenu;
	QPopupMenu *connectionMenu;
	QPopupMenu *helpMenu_;
	QLabel *status;
	QTimer *pollTimer;
	KMasqDock *dock;
	KMasqServerInfo *serverInfo;
	KMasqConnectionInfo *connectionInfo;
	QLabel *connectedTime;
	int serverTime;
	int connectTime;
	QFrame *frameOutput;
	QFrame *frameInput;
	float intInput[63];
	float intOutput[63];
	float packetsIn, packetsOut, bytesIn, lastBytesIn, bytesOut, lastBytesOut, speed;
	QLabel *netLoadIn;
	QLabel *netLoadOut;
	bool dialLock;
	bool killLock;
	int cCounter;
	//QLineEdit *edit;
	
public slots:
	/** Show the default About Qt box */
	void slotServerConnect();
	void slotServerDisconnect();
	void slotServerChangeUser();
	void slotServerInfo();
	void slotServerHistory();
	void slotServerLicense();
	void slotServerConfigure();
	void slotConnectionDial();
	void slotConnectionHangup();
	void slotConnectionInfo();
	void slotConnectionLockDial();
	void slotConnectionLockKill();
	/** Closes the app **/
	void slotServerQuit();
	void slotSetStatusMsg(const char *);
	void slotAddConnection(const char *);
	void slotUpdateStats();
	void slotSetStatus(const char *);
	void slotGotInfo(QString, QString);
	void slotAuthenticated(QString, QString);
	
signals:
	void setDockStatus(const char *);
	void sigUpdateDock(float, float, float, float);
	void sigSetToolTip(const char *);
	
};

#endif
