/***************************************************************************
                          kmasqdock.cpp  -  description
                             -------------------
    begin                : Thu Mar 22 2001
    copyright            : (C) 2001 by Stephan de Georgio
    email                : kmasqdialer@stephan.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmasqdock.h"

#include <klocale.h>
#include <qstring.h>
#include <qpainter.h>
#include <qcursor.h>

const int pixelsx[16] = {8, 9, 10, 11, 12, 13, 14, 14, 13, 12, 11, 10, 9, 8, 7, 7};
const int pixelsy[16] = {14, 14, 14, 14, 14, 14, 15, 16, 17, 17, 17, 17, 17, 17, 16, 15};

KMasqDock::KMasqDock()
{
	iconLoader = new KIconLoader();
	setPixmap(iconLoader->loadIcon("kmasqdialer-dock1", KIcon::MainToolbar));
	contextMenu()->insertItem(i18n("&Hide"), 1001);
	contextMenu()->insertSeparator();
	contextMenu()->insertItem(i18n("&Dial"), 1002);
	contextMenu()->setItemEnabled(1002, false);
	contextMenu()->connectItem(1001, this, SLOT(slotHide()));
	contextMenu()->connectItem(1002, this, SLOT(slotDial()));
	dialed = false;
	connect(contextMenu(), SIGNAL(activated(int)), this, SLOT(slotItemSelected(int)));
	bytesIn = lastBytesIn = bytesOut = lastBytesOut = 0;
	installEventFilter(this);
	
	dialTimer = new QTimer;
	connect(dialTimer, SIGNAL(timeout()), this, SLOT(slotDialTimer()));
	dialTimer->stop();
	timerCounter = 0;
}

KMasqDock::~KMasqDock()
{
}

void KMasqDock::slotHide()
{
	if(contextMenu()->text(1001) == i18n("&Hide"))
	{
		contextMenu()->changeItem(1001, i18n("&Show"));
		emit sigChangeStatus("hide");
	}
	else
	{
		contextMenu()->changeItem(1001, i18n("&Hide"));
		emit sigChangeStatus("show");
	}
}

void KMasqDock::slotDial()
{
	if(contextMenu()->text(1002) == i18n("&Dial"))
	{
		emit sigChangeStatus("dial");
	}
	else
	{
		emit sigChangeStatus("hangup");
	}
}

void KMasqDock::slotSetStatus(const char *status)
{
	QString temp(status);
	if(temp == "dialed")
	{
		updateStuff();
		dialTimer->stop();
		contextMenu()->setItemEnabled(1002, true);
		contextMenu()->changeItem(1002, i18n("&Hang Up"));
		if(dialed == false)
		{
			setPixmap(iconLoader->loadIcon("kmasqdialer-dock2", KIcon::MainToolbar));
		}
		dialed = true;
	}
	if(temp == "hungup")
	{
		updateStuff();
		dialTimer->stop();
		contextMenu()->setItemEnabled(1002, true);
		contextMenu()->changeItem(1002, i18n("&Dial"));
		if(dialed == true)
		{
			setPixmap(iconLoader->loadIcon("kmasqdialer-dock1", KIcon::MainToolbar));
		}
		dialed = false;
		bytesIn = lastBytesIn = bytesOut = lastBytesOut = 0;
	}
	if(temp == "connected")
	{
		contextMenu()->setItemEnabled(1002, true);
		contextMenu()->changeItem(1002, i18n("&Dial"));
	}
	if(temp == "disconnected")
	{
		contextMenu()->setItemEnabled(1002, false);
		contextMenu()->changeItem(1002, i18n("&Dial"));
		if(dialed == true)
		{
			setPixmap(iconLoader->loadIcon("kmasqdialer-dock1", KIcon::MainToolbar));
		}
		dialed = false;
	}
	if(temp == "minimized")
	{
		contextMenu()->changeItem(1001, i18n("&Show"));
	}
	if(temp == "dialing")
	{
		dialTimer->start(100);
	}
}

void KMasqDock::slotItemSelected(int id)
{
	if(id == -18)
		emit sigChangeStatus("quit");
}

void KMasqDock::mousePressEvent(QMouseEvent * event)
{
	if(event->button() == QMouseEvent::LeftButton)
	{
		slotHide();
	}
	if(event->button() == QMouseEvent::RightButton)
	{
		contextMenu()->exec(QCursor::pos());
	}
}

void KMasqDock::slotUpdateDock(float bytesInTemp, float lastBytesInTemp, float bytesOutTemp, float lastBytesOutTemp)
{
	bytesIn = bytesInTemp;
	lastBytesIn = lastBytesInTemp;
	bytesOut = bytesOutTemp;
	lastBytesOut = lastBytesOutTemp;
	updateStuff();
}

bool KMasqDock::eventFilter(QObject *o, QEvent *e)
{
	
	if(o == this && e->type() == QEvent::Paint)
	{
		updateStuff();
	}
	return false;
}

void KMasqDock::updateStuff()
{
	QPixmap *pixDock = new QPixmap;
	pixDock->resize(2, 2);
	QPainter *paintDock = new QPainter(pixDock);
	paintDock->setBackgroundColor(red);
	paintDock->eraseRect(0, 0, 2, 2);
	if(bytesIn != lastBytesIn)
	{
		paintDock->setPen(Qt::green);
		paintDock->drawRect(0, 0, 2, 2);
	}
	paintDock->flush();
	bitBlt(this, 4, 19, pixDock, 0, 0, 2, 2);
		
	paintDock->eraseRect(0, 0, 2, 2);
	if(bytesOut != lastBytesOut)
	{
		paintDock->setPen(Qt::green);
		paintDock->drawRect(0, 0, 2, 2);
	}
	paintDock->flush();
	bitBlt(this, 16, 19, pixDock, 0, 0, 2, 2);
	delete paintDock;
	delete pixDock;
	
	if(dialTimer->isActive())
	{
		QPixmap *pixDial = new QPixmap;
		pixDial->resize(1, 1);
		QPainter *paintDial = new QPainter(pixDial);
		paintDial->setPen(Qt::black);
		paintDial->drawPoint(0,0);
		bitBlt(this, pixelsx[timerCounter], pixelsy[timerCounter], pixDial, 0, 0, 1, 1);
		paintDial->setPen(Qt::white);
		paintDial->drawPoint(0,0);
		if(timerCounter > 0)
			bitBlt(this, pixelsx[timerCounter - 1], pixelsy[timerCounter - 1], pixDial, 0, 0, 1, 1);
		else
			bitBlt(this, pixelsx[15], pixelsy[15], pixDial, 0, 0, 1, 1);
		
		if(timerCounter == 15)
			timerCounter = -1;
			timerCounter++;
			
		delete paintDial;
		delete pixDial;
	}
	else
	{
		QPixmap *pixDial = new QPixmap;
		pixDial->resize(1, 1);
		QPainter *paintDial = new QPainter(pixDial);
		paintDial->setPen(Qt::white);
		paintDial->drawPoint(0,0);
		if(timerCounter>0)
			bitBlt(this, pixelsx[timerCounter-1], pixelsy[timerCounter-1], pixDial, 0, 0, 1, 1);
		else
			bitBlt(this, pixelsx[16], pixelsy[16], pixDial, 0, 0, 1, 1);
		delete paintDial;
		delete pixDial;
	}
}

void KMasqDock::slotDialTimer()
{
	updateStuff();
}

void KMasqDock::slotSetToolTip(const char *text)
{
	QToolTip::add(this, text);
}
